package com.sun.xml.internal.fastinfoset.sax;

import java.io.File;

public class SystemIdResolver
{
    public SystemIdResolver() {
    }
    
    public static String getAbsoluteURIFromRelative(String localPath) {
        if (localPath == null || localPath.length() == 0) {
            return "";
        }
        String absolutePath = localPath;
        if (!isAbsolutePath(localPath)) {
            try {
                absolutePath = getAbsolutePathFromRelativePath(localPath);
            }
            catch (SecurityException se) {
                return "file:" + localPath;
            }
        }
        String urlString;
        if (null != absolutePath) {
            urlString = (absolutePath.startsWith(File.separator) ? ("file://" + absolutePath) : ("file:///" + absolutePath));
        }
        else {
            urlString = "file:" + localPath;
        }
        return replaceChars(urlString);
    }
    
    private static String getAbsolutePathFromRelativePath(String relativePath) {
        return new File(relativePath).getAbsolutePath();
    }
    
    public static boolean isAbsoluteURI(String systemId) {
        if (systemId == null) {
            return false;
        }
        if (isWindowsAbsolutePath(systemId)) {
            return false;
        }
        int fragmentIndex = systemId.indexOf(35);
        int queryIndex = systemId.indexOf(63);
        int slashIndex = systemId.indexOf(47);
        int colonIndex = systemId.indexOf(58);
        int index = systemId.length() - 1;
        if (fragmentIndex > 0) {
            index = fragmentIndex;
        }
        if (queryIndex > 0 && queryIndex < index) {
            index = queryIndex;
        }
        if (slashIndex > 0 && slashIndex < index) {
            index = slashIndex;
        }
        return colonIndex > 0 && colonIndex < index;
    }
    
    public static boolean isAbsolutePath(String systemId) {
        if (systemId == null) {
            return false;
        }
        File file = new File(systemId);
        return file.isAbsolute();
    }
    
    private static boolean isWindowsAbsolutePath(String systemId) {
        return isAbsolutePath(systemId) && (systemId.length() > 2 && systemId.charAt(1) == ':' && Character.isLetter(systemId.charAt(0)) && (systemId.charAt(2) == '\\' || systemId.charAt(2) == '/'));
    }
    
    private static String replaceChars(String str) {
        StringBuffer buf = new StringBuffer(str);
        for (int length = buf.length(), i = 0; i < length; ++i) {
            char currentChar = buf.charAt(i);
            if (currentChar == ' ') {
                buf.setCharAt(i, '%');
                buf.insert(i + 1, "20");
                length += 2;
                i += 2;
            }
            else if (currentChar == '\\') {
                buf.setCharAt(i, '/');
            }
        }
        return buf.toString();
    }
    
    public static String getAbsoluteURI(String systemId) {
        String absoluteURI = systemId;
        if (isAbsoluteURI(systemId)) {
            if (systemId.startsWith("file:")) {
                String str = systemId.substring(5);
                if (str != null && str.startsWith("/")) {
                    if (str.startsWith("///") || !str.startsWith("//")) {
                        int secondColonIndex = systemId.indexOf(58, 5);
                        if (secondColonIndex > 0) {
                            String localPath = systemId.substring(secondColonIndex - 1);
                            try {
                                if (!isAbsolutePath(localPath)) {
                                    absoluteURI = systemId.substring(0, secondColonIndex - 1) + getAbsolutePathFromRelativePath(localPath);
                                }
                            }
                            catch (SecurityException se) {
                                return systemId;
                            }
                        }
                    }
                    return replaceChars(absoluteURI);
                }
                else {
                    return getAbsoluteURIFromRelative(systemId.substring(5));
                }
            }
            else {
                return systemId;
            }
        }
        else {
            return getAbsoluteURIFromRelative(systemId);
        }
    }
}
